use crate::util::surrealdb::schemas;
use leptos::*;
use leptos_use::*;

use super::ParticipantsAction;

pub fn init_websocket() {
    let UseWebsocketReturn {
        ready_state,
        message,
        ..
    } = use_websocket("ws://localhost:3000/ws");

    create_effect(move |prev_value| {
        let msg = message.get();

        if prev_value != Some(msg.clone()) {
            match msg {
                Some(ref text) => handle_message(text),
                None => (),
            }
        }

        msg
    });
}

fn handle_message(message: &str) {
    let action: ParticipantsAction = match serde_json::from_str(message) {
        Ok(res) => res,
        Err(_err) => {
            return;
        }
    };

    let participants_context = use_context::<RwSignal<Vec<schemas::Participant>>>().unwrap();

    match action {
        ParticipantsAction::Add { participant } => {
            participants_context.update(|participants| participants.push(participant))
        }
        ParticipantsAction::Remove { id } => participants_context
            .update(|participants| participants.retain(|participant| participant.id != id)),
        ParticipantsAction::Replace { participant } => {
            participants_context.update(|participants| {
                if let Some(index) = participants
                    .iter()
                    .position(|item| item.id == participant.id)
                {
                    let _ = std::mem::replace(&mut participants[index], participant);
                }
            })
        }
    }
}
